% Script for generating contextualized iMT1026 v 3.0 under different
% oxygention conditions (requires COBRA Toolbox v3)
% -------------------------- Pedro Saa UC 2019 ----------------------------
% initCobraToolbox

% Read original iMT1026v3 GSMM
model = readCbModel('iMT1026v3.xml');

% Add thaumatin reaction
model = addReaction(model,'thauDNA','0.819 damp[c] + 0.976 dtmp[c] + 0.769 dgmp[c] + 0.681 dcmp[c] + 11.031 atp[c] + 7.786 h2o[c] <=>  11.031 h[c] + 11.031 pi[c] + 11.031 adp[c] + 1 Thaudna[c]');
model = addReaction(model,'thauRNA','0.940 amp[c] + 0.789 ump[c] + 0.656 gmp[c] + 0.740 cmp[c] + 7.499 atp[c] + 4.374 h2o[c] <=>  7.499 h[c] + 7.499 pi[c] + 7.499 adp[c] + 1 Thaurna[c]');
model = addReaction(model,'thauAA','0.748 ala_L[c] + 0.512 arg_L[c] + 0.354 asn_L[c] + 0.551 asp_L[c] + 0.669 cys_L[c] + 0.354 glu_L[c] + 0.157 gln_L[c] + 0.945 gly[c] + 0.315 ile_L[c] + 0.669 leu_L[c] + 0.433 lys_L[c] + 0.079 met_L[c] + 0.630 phe_L[c] + 0.512 pro_L[c] + 0.591 ser_L[c] + 0.906 thr_L[c] + 0.118 trp_L[c] + 0.315 tyr_L[c] + 0.394 val_L[c] + 39.786 atp[c] + 39.786 h2o[c]  <=> 39.786 h[c] + 39.786 pi[c] + 39.786 adp[c] + 1 Thauaa[c]');
model = addReaction(model,'pThau','0.997 Thauaa[c] + 0.003 Thaurna[c] + 2.84e-005 Thaudna[c] ->  1 Thau[c]');
model = addReaction(model,'thaut',' Thau[c] -> Thau[e]');
model = addReaction(model,'Ex_thau',' Thau[e] -> ');

% Add contextualized constraints
[bounds,rx_IDs] = xlsread('list_of_constraints.xlsx',1,'A2:C278');
model = changeRxnBounds(model,rx_IDs,bounds(:,1),'l');
model = changeRxnBounds(model,rx_IDs,bounds(:,2),'u');

% Extract biomass reaction
bio_rxn_index = 1305;
bio_rxn = model.S(:,bio_rxn_index);

% Definition of GAME coefficients
s_GAME_L = 94.7895431213887;    % mmol ATP/gDCW/h (GAME)
s_GAME_M = 75.2676922735921;    % mmol ATP/gDCW/h (GAME)
s_GAME_E = 65.72310225;         % mmol ATP/gDCW/h (GAME)

% Definition of biomass reaction stoichiometries
% L condition
bio_rxn_L = bio_rxn;
bio_rxn_L(49) = -s_GAME_L;
bio_rxn_L(50) =  s_GAME_L;
bio_rxn_L(4)  = -s_GAME_L;
bio_rxn_L(6)  =  s_GAME_L;
bio_rxn_L(7)  =  s_GAME_L;

% M condition
bio_rxn_M = bio_rxn;
bio_rxn_M(49) = -s_GAME_M;
bio_rxn_M(50) =  s_GAME_M;
bio_rxn_M(4)  = -s_GAME_M;
bio_rxn_M(6)  =  s_GAME_M;
bio_rxn_M(7)  =  s_GAME_M;

% E condition
bio_rxn_E = bio_rxn;
bio_rxn_E(49) = -s_GAME_E;
bio_rxn_E(50) =  s_GAME_E;
bio_rxn_E(4)  = -s_GAME_E;
bio_rxn_E(6)  =  s_GAME_E;
bio_rxn_E(7)  =  s_GAME_E;

% Add corresponding fields
model.S = [model.S,bio_rxn_L,bio_rxn_M,bio_rxn_E];
model.rxns(end+1:end+3)     = {'BIOMASS L';'BIOMASS M';'BIOMASS E'};
model.rxnNames(end+1:end+3) = {'biomass glucose growth L condition';'biomass glucose growth M condition';'biomass glucose growth E condition'};
model.lb(end+1:end+3) = 0;
model.ub(end+1:end+3) = 1e3;
model.c = zeros(numel(model.rxns),1);
model.rules(end+1:end+3)        = {'';'';''};
model.rxnECNumbers(end+1:end+3) = {'';'';''};
model.subSystems(end+1:end+3)   = {'';'';''};
model.grRules(end+1:end+3)      = {'';'';''};
model.modelName                 = [model.modelName,' glucose growth under different oxygenation levels'];

% Save final model (MAT)
save iMT1026v3_contextualized.mat  '-mat' model

% Save final model (SBML)
clearvars -except model
model = writeCbModel(model,'format','sbml','fileName','iMT1026v3_contextualized.xml');
